
/*! Copyright (C) [2011] [David Mainzer]
 * 
 *  This program is free software; you can redistribute it and/or modify it under the terms 
 *  of the GNU General Public License as published by the Free Software Foundation; either 
 *  version 3 of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU General Public License for more details.
 *  
 *	You should have received a copy of the GNU General Public License along with this program; 
 *	if not, see <http://www.gnu.org/licenses/>.  
 */


/**  Wenn etwas in dieser Klasse fehlt, fuegt dies ruhig hinzu :)
 *
 *  N      N      N
 *    \    |    /
 *      2  |  2
 *        \|/
 *  N -1-  P  -1- N 
 *        /|\
 *      2  |  2
 *    /    |    \
 *  N      N      N
 *  
 *  WITH:
 *  P = MassPoint
 *  N = Neighbors
 *  1 = structural springs
 *  2 = shear springs
 */

public class MassPoint {
	// MassPoint ID (given by the wrl-file from coordIndex)
	private int ID;
	
	// store the OLD position
	private Vector3 oldPos = new Vector3();
	// store the CURRENT position
	private Vector3 curPos = new Vector3();
	// store the NEW position (adding force to curPos)
	private Vector3 newPos = new Vector3();
	
	// mass of MassPoint
	private final float mass;

	// force affect to MassPoint
	private Vector3 force = new Vector3();
	
	private Vector3 velocity = new Vector3();
	
	private boolean movable = false;
	
	// constructors
	public MassPoint( int ID )
	{
		this.ID     = ID;
		this.mass = 0.0005F;
	}
	
	public MassPoint( int ID, float mass )
	{
		this.ID     = ID;
		this.mass   = mass;
	}
	
	public MassPoint( int ID, float x, float y, float z )
	{
		this.ID     = ID;

		this.oldPos.set(x, y, z);
		
		this.newPos.set(x, y, z);

		this.curPos.set(x, y, z);
		
		this.mass = 0.00005F;
	}
	
	public MassPoint( int ID, float x, float y, float z, float mass )
	{
		this.ID     = ID;

		this.oldPos.set(x, y, z);
		
		this.newPos.set(x, y, z);
		
		this.curPos.set(x, y, z);
		
		this.mass = mass;
	}
	
	public MassPoint( int ID, float [] pos )
	{
		if (pos.length != 3)
		{
			System.err.println("pos vector has " + pos.length + "elements but only 3 accepted.");
			System.exit(-1);
		}
		this.ID     = ID;
		this.oldPos.set(pos[0], pos[1], pos[2]);
		
		this.newPos.set(pos[0], pos[1], pos[2]);
		
		this.curPos.set(pos[0], pos[1], pos[2]);
		
		this.mass = 0.0005F;
	}
	
	public MassPoint( int ID, float [] pos, float mass )
	{
		if (pos.length != 3)
		{
			System.err.println("pos vector has " + pos.length + "elements but only 3 accepted.");
			System.exit(-1);
		}
		this.ID     = ID;
		this.oldPos.set(pos[0], pos[1], pos[2]);
		
		this.newPos.set(pos[0], pos[1], pos[2]);
		
		this.curPos.set(pos[0], pos[1], pos[2]);

		this.mass = mass;
	}
	
	/** add force to the force vector of the mass-point
	 * 
	 * @param force
	 */
	public void addForce( Vector3 force2 )
	{
		if(movable)
		{
			Vector3 tmp = this.force.add(force2);
			this.force = tmp;
		}		
	}
	
	public void addForce( float forceX, float forceY, float forceZ )
	{
			Vector3 tmp = this.force.add(forceX, forceY, forceZ);
			this.force = tmp;
	}
	
	
	/** update current position by adding the force and store as new position, create a new empty force vector
	 *  and set new as current position
	 */
	public void updateCurPos ( float fFrameTime )
	{
		if(movable)
		{
			this.velocity = this.velocity.add( this.force.div( this.mass ).mul( fFrameTime ) );
			// etwas dmpfung fr die geschwindigkeit kann ja nicht schaden ;)
			this.velocity = this.velocity.mul(0.999f);
		
			this.newPos = this.curPos.add( this.velocity.mul( fFrameTime ) );
		}
		this.force = new Vector3();
		this.curPos = this.newPos;
	}
	
	public void updateCurPos ()
	{
		if(movable)
		{
			this.newPos = this.curPos.add(this.force);
		}
		this.force = new Vector3();
		this.curPos = this.newPos;
	}
	
	public Vector3 getVelocity( float fFrameTime )
	{
		return velocity;
	}

///////////////////////
// GETTER and SETTER //
///////////////////////
	
	/** return the id of the MassPoint
	 *
	 * @return ID -- MassPoint ID
	 */
	public int getID()
	{
		return this.ID;
	}

	/** return the current position of this MassPoint
	 *
	 * @return curPos -- the current position of MassPoint
	 */
	public Vector3 getCurPos()
	{
		return this.curPos;
	}
	
	/** return the x coordinate of the current position of this MassPoint
	 *
	 * @return curPos.x() -- the x coordinate of the current position
	 */
	public float getX()
	{
		return this.curPos.xyz[0];
	}
	
	/** return the y coordinate of the current position of this MassPoint
	 *
	 * @return curPos.y() -- the y coordinate of the current position
	 */
	public float getY()
	{
		return this.curPos.xyz[1];
	}
	
	/** return the z coordinate of the current position of this MassPoint
	 *
	 * @return curPos.z() -- the z coordinate of the current position
	 */
	public float getZ()
	{
		return this.curPos.xyz[2];
	}
	
	/** overwrite the current position (have you backup the current position as old position)
	 *
	 * @warning backup the current position as old position before
	 * @param x -- new x coordinate
	 * @param y -- new y coordinate
	 * @param z -- new z coordinate
	 */
	public void setCurPos(float x, float y, float z)
	{
		this.curPos.set(x, y, z);
	}
	
	/** backup the current position as old position
	 *
	 */
	public void setOldPos( )
	{
		this.oldPos = this.curPos;
	}
	
	/**
	 * 
	 * @return the mass of this massPoint
	 */
	public float getMass()
	{
		return this.mass;
	}
	
	public boolean getMove()
	{
		return this.movable;
	}
	
	public void setMove(boolean value)
	{
		this.movable = value;
	}
	
	/** return the movement of MassPoint
	 *
	 * @return oldPos.sub(curPos) -- the vector of movement
	 */
	public Vector3 getMovement()
	{
		return this.oldPos.sub(curPos);
	}
	
	public Vector3 getForce()
	{
		return this.force;
	}
}
